@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2006. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2006-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -12,12 +12,14 @@
 #define BOOST_INTERPROCESS_TEST_MEMORY_ALGORITHM_TEST_TEMPLATE_HEADER
 
 #include <boost/interprocess/detail/config_begin.hpp>
+
+#include <boost/container/vector.hpp>
+
 #include <vector>
 #include <iostream>
-#include <new>
-#include <utility>
+#include <new> //std::nothrow
 #include <cstring>   //std::memset
-#include <boost/interprocess/containers/vector.hpp>
+#include <typeinfo>
 
 namespace boost { namespace interprocess { namespace test {
 
@@ -32,13 +34,13 @@ bool test_allocation(Allocator &a)
       ; t != EndDeallocationType
       ; t = (deallocation_type)((int)t + 1)){
       std::vector<void*> buffers;
-	  typename Allocator::size_type free_memory = a.get_free_memory();
+      typename Allocator::size_type free_memory = a.get_free_memory();
 
-      for(int i = 0; true; ++i){
+      for(std::size_t i = 0; true; ++i){
          void *ptr = a.allocate(i, std::nothrow);
          if(!ptr)
             break;
-		 std::size_t size = a.size(ptr);
+       std::size_t size = a.size(ptr);
          std::memset(ptr, 0, size);
          buffers.push_back(ptr);
       }
@@ -46,7 +48,7 @@ bool test_allocation(Allocator &a)
       switch(t){
          case DirectDeallocation:
          {
-            for(int j = 0, max = (int)buffers.size()
+            for(std::size_t j = 0, max = buffers.size()
                ;j < max
                ;++j){
                a.deallocate(buffers[j]);
@@ -55,7 +57,7 @@ bool test_allocation(Allocator &a)
          break;
          case InverseDeallocation:
          {
-            for(int j = (int)buffers.size()
+            for(std::size_t j = buffers.size()
                ;j--
                ;){
                a.deallocate(buffers[j]);
@@ -64,19 +66,19 @@ bool test_allocation(Allocator &a)
          break;
          case MixedDeallocation:
          {
-            for(int j = 0, max = (int)buffers.size()
+            for(std::size_t j = 0, max = buffers.size()
                ;j < max
                ;++j){
-               int pos = (j%4)*((int)buffers.size())/4;
+               std::size_t pos = (j%4)*(buffers.size())/4;
                a.deallocate(buffers[pos]);
-               buffers.erase(buffers.begin()+pos);
+               buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
             }
          }
          break;
          default:
          break;
       }
-      bool ok = free_memory == a.get_free_memory() && 
+      bool ok = free_memory == a.get_free_memory() &&
                a.all_memory_deallocated() && a.check_sanity();
       if(!ok)  return ok;
    }
@@ -90,42 +92,50 @@ template<class Allocator>
 bool test_allocation_shrink(Allocator &a)
 {
    std::vector<void*> buffers;
+   std::vector<std::size_t> sizes;
 
    //Allocate buffers with extra memory
-   for(int i = 0; true; ++i){
+   for(std::size_t i = 0; true; ++i){
       void *ptr = a.allocate(i*2, std::nothrow);
       if(!ptr)
          break;
-	  std::size_t size = a.size(ptr);
+      std::size_t size = a.size(ptr);
       std::memset(ptr, 0, size);
       buffers.push_back(ptr);
+      sizes.push_back(size);
    }
 
    //Now shrink to half
-   for(int i = 0, max = (int)buffers.size()
+   for(std::size_t i = 0, max = buffers.size()
       ;i < max
       ; ++i){
       typename Allocator::size_type received_size;
+      char *reuse = static_cast<char*>(buffers[i]);
       if(a.template allocation_command<char>
-         ( boost::interprocess::shrink_in_place | boost::interprocess::nothrow_allocation, i*2
-         , i, received_size, static_cast<char*>(buffers[i])).first){
-         if(received_size > std::size_t(i*2)){
+         ( boost::interprocess::shrink_in_place | boost::interprocess::nothrow_allocation, sizes[i]
+         , received_size = i, reuse)){
+         if(received_size > sizes[i]){
             return false;
          }
          if(received_size < std::size_t(i)){
             return false;
          }
-		 std::memset(buffers[i], 0, a.size(buffers[i]));
+         const std::size_t sz = a.size(buffers[i]);
+         if (received_size != sz) {
+            return false;
+         }
+
+         std::memset(buffers[i], 0, sz);
       }
    }
-   
+
    //Deallocate it in non sequential order
-   for(int j = 0, max = (int)buffers.size()
+   for(std::size_t j = 0, max = buffers.size()
       ;j < max
       ;++j){
-      int pos = (j%4)*((int)buffers.size())/4;
+      std::size_t pos = (j%4)*(buffers.size())/4;
       a.deallocate(buffers[pos]);
-      buffers.erase(buffers.begin()+pos);
+      buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
    }
 
    return a.all_memory_deallocated() && a.check_sanity();
@@ -140,17 +150,17 @@ bool test_allocation_expand(Allocator &a)
    std::vector<void*> buffers;
 
    //Allocate buffers with extra memory
-   for(int i = 0; true; ++i){
+   for(std::size_t i = 0; true; ++i){
       void *ptr = a.allocate(i, std::nothrow);
       if(!ptr)
          break;
-	  std::size_t size = a.size(ptr);
+     std::size_t size = a.size(ptr);
       std::memset(ptr, 0, size);
       buffers.push_back(ptr);
    }
 
    //Now try to expand to the double of the size
-   for(int i = 0, max = (int)buffers.size()
+   for(std::size_t i = 0, max = buffers.size()
       ;i < max
       ;++i){
       typename Allocator::size_type received_size;
@@ -158,26 +168,27 @@ bool test_allocation_expand(Allocator &a)
       std::size_t preferred_size = i*2;
       preferred_size = min_size > preferred_size ? min_size : preferred_size;
 
+      char *reuse = static_cast<char*>(buffers[i]);
       while(a.template allocation_command<char>
          ( boost::interprocess::expand_fwd | boost::interprocess::nothrow_allocation, min_size
-         , preferred_size, received_size, static_cast<char*>(buffers[i])).first){
+         , received_size = preferred_size, reuse)){
          //Check received size is bigger than minimum
          if(received_size < min_size){
             return false;
          }
          //Now, try to expand further
-		 min_size       = received_size+1;
+       min_size       = received_size+1;
          preferred_size = min_size*2;
       }
    }
-   
+
    //Deallocate it in non sequential order
-   for(int j = 0, max = (int)buffers.size()
+   for(std::size_t j = 0, max = buffers.size()
       ;j < max
       ;++j){
-      int pos = (j%4)*((int)buffers.size())/4;
+      std::size_t pos = (j%4)*(buffers.size())/4;
       a.deallocate(buffers[pos]);
-      buffers.erase(buffers.begin()+pos);
+      buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
    }
 
    return a.all_memory_deallocated() && a.check_sanity();
@@ -194,13 +205,14 @@ bool test_allocation_shrink_and_expand(Allocator &a)
    std::vector<bool>        size_reduced;
 
    //Allocate buffers wand store received sizes
-   for(int i = 0; true; ++i){
+   for(std::size_t i = 0; true; ++i){
       typename Allocator::size_type received_size;
+      char *reuse = 0;
       void *ptr = a.template allocation_command<char>
-         ( boost::interprocess::allocate_new | boost::interprocess::nothrow_allocation, i, i*2, received_size).first;
+         ( boost::interprocess::allocate_new | boost::interprocess::nothrow_allocation, i, received_size = i*2, reuse);
       if(!ptr){
          ptr = a.template allocation_command<char>
-            ( boost::interprocess::allocate_new | boost::interprocess::nothrow_allocation, 1, i*2, received_size).first;
+            ( boost::interprocess::allocate_new | boost::interprocess::nothrow_allocation, 1, received_size = i*2, reuse);
          if(!ptr)
             break;
       }
@@ -209,13 +221,14 @@ bool test_allocation_shrink_and_expand(Allocator &a)
    }
 
    //Now shrink to half
-   for(int i = 0, max = (int)buffers.size()
+   for(std::size_t i = 0, max = buffers.size()
       ; i < max
       ; ++i){
       typename Allocator::size_type received_size;
+      char *reuse = static_cast<char*>(buffers[i]);
       if(a.template allocation_command<char>
          ( boost::interprocess::shrink_in_place | boost::interprocess::nothrow_allocation, received_sizes[i]
-         , i, received_size, static_cast<char*>(buffers[i])).first){
+         , received_size = i, reuse)){
          if(received_size > std::size_t(received_sizes[i])){
             return false;
          }
@@ -227,14 +240,15 @@ bool test_allocation_shrink_and_expand(Allocator &a)
    }
 
    //Now try to expand to the original size
-   for(int i = 0, max = (int)buffers.size()
+   for(std::size_t i = 0, max = buffers.size()
       ;i < max
       ;++i){
       typename Allocator::size_type received_size;
       std::size_t request_size = received_sizes[i];
+      char *reuse = static_cast<char*>(buffers[i]);
       if(a.template allocation_command<char>
          ( boost::interprocess::expand_fwd | boost::interprocess::nothrow_allocation, request_size
-         , request_size, received_size, static_cast<char*>(buffers[i])).first){
+         , received_size = request_size, reuse)){
          if(received_size != received_sizes[i]){
             return false;
          }
@@ -243,14 +257,14 @@ bool test_allocation_shrink_and_expand(Allocator &a)
          return false;
       }
    }
-   
+
    //Deallocate it in non sequential order
-   for(int j = 0, max = (int)buffers.size()
+   for(std::size_t j = 0, max = buffers.size()
       ;j < max
       ;++j){
-      int pos = (j%4)*((int)buffers.size())/4;
+      std::size_t pos = (j%4)*(buffers.size())/4;
       a.deallocate(buffers[pos]);
-      buffers.erase(buffers.begin()+pos);
+      buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
    }
 
    return a.all_memory_deallocated() && a.check_sanity();
@@ -266,7 +280,7 @@ bool test_allocation_deallocation_expand(Allocator &a)
    std::vector<void*> buffers;
 
    //Allocate buffers with extra memory
-   for(int i = 0; true; ++i){
+   for(std::size_t i = 0; true; ++i){
       void *ptr = a.allocate(i, std::nothrow);
       if(!ptr)
          break;
@@ -277,7 +291,7 @@ bool test_allocation_deallocation_expand(Allocator &a)
 
    //Now deallocate the half of the blocks
    //so expand maybe can merge new free blocks
-   for(int i = 0, max = (int)buffers.size()
+   for(std::size_t i = 0, max = buffers.size()
       ;i < max
       ;++i){
       if(i%2){
@@ -287,7 +301,7 @@ bool test_allocation_deallocation_expand(Allocator &a)
    }
 
    //Now try to expand to the double of the size
-   for(int i = 0, max = (int)buffers.size()
+   for(std::size_t i = 0, max = buffers.size()
       ;i < max
       ;++i){
       //
@@ -297,9 +311,10 @@ bool test_allocation_deallocation_expand(Allocator &a)
          std::size_t preferred_size = i*2;
          preferred_size = min_size > preferred_size ? min_size : preferred_size;
 
+         char *reuse = static_cast<char*>(buffers[i]);
          while(a.template allocation_command<char>
             ( boost::interprocess::expand_fwd | boost::interprocess::nothrow_allocation, min_size
-            , preferred_size, received_size, static_cast<char*>(buffers[i])).first){
+            , received_size = preferred_size, reuse)){
             //Check received size is bigger than minimum
             if(received_size < min_size){
                return false;
@@ -310,18 +325,18 @@ bool test_allocation_deallocation_expand(Allocator &a)
          }
       }
    }
-   
+
    //Now erase null values from the vector
    buffers.erase( std::remove(buffers.begin(), buffers.end(), static_cast<void*>(0))
                 , buffers.end());
 
    //Deallocate it in non sequential order
-   for(int j = 0, max = (int)buffers.size()
+   for(std::size_t j = 0, max = buffers.size()
       ;j < max
       ;++j){
-      int pos = (j%4)*((int)buffers.size())/4;
+      std::size_t pos = (j%4)*(buffers.size())/4;
       a.deallocate(buffers[pos]);
-      buffers.erase(buffers.begin()+pos);
+      buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
    }
 
    return a.all_memory_deallocated() && a.check_sanity();
@@ -337,11 +352,11 @@ template<class Allocator>
 bool test_allocation_with_reuse(Allocator &a)
 {
    //We will repeat this test for different sized elements
-   for(int sizeof_object = 1; sizeof_object < 20; ++sizeof_object){
+   for(std::size_t sizeof_object = 1; sizeof_object < 20u; ++sizeof_object){
       std::vector<void*> buffers;
 
       //Allocate buffers with extra memory
-      for(int i = 0; true; ++i){
+      for(std::size_t i = 0; true; ++i){
          void *ptr = a.allocate(i*sizeof_object, std::nothrow);
          if(!ptr)
             break;
@@ -349,10 +364,10 @@ bool test_allocation_with_reuse(Allocator &a)
          std::memset(ptr, 0, size);
          buffers.push_back(ptr);
       }
-      
+
       //Now deallocate all except the latest
       //Now try to expand to the double of the sizeof_object
-      for(int i = 0, max = (int)buffers.size() - 1
+      for(std::size_t i = 0, max = buffers.size() - 1
          ;i < max
          ;++i){
          a.deallocate(buffers[i]);
@@ -364,24 +379,25 @@ bool test_allocation_with_reuse(Allocator &a)
 
       //Now allocate with reuse
       typename Allocator::size_type received_size = 0;
-      for(int i = 0; true; ++i){
+      for(std::size_t i = 0; true; ++i){
          std::size_t min_size = (received_size + 1);
          std::size_t prf_size = (received_size + (i+1)*2);
-         std::pair<void*, bool> ret = a.raw_allocation_command
+         void *reuse = ptr;
+         void *ret = a.raw_allocation_command
             ( boost::interprocess::expand_bwd | boost::interprocess::nothrow_allocation, min_size
-            , prf_size, received_size, static_cast<char*>(ptr), sizeof_object);
-         if(!ret.first)
+            , received_size = prf_size, reuse, sizeof_object);
+         if(!ret)
             break;
          //If we have memory, this must be a buffer reuse
-         if(!ret.second)
+         if(!reuse)
             return 1;
          if(received_size < min_size)
             return 1;
-         ptr = ret.first;
+         ptr = ret;
       }
       //There is only a single block so deallocate it
       a.deallocate(ptr);
-      
+
       if(!a.all_memory_deallocated() || !a.check_sanity())
          return false;
    }
@@ -397,17 +413,18 @@ bool test_aligned_allocation(Allocator &a)
    //Allocate aligned buffers in a loop
    //and then deallocate it
    bool continue_loop = true;
-   for(unsigned int i = 1; continue_loop; i <<= 1){
-      for(unsigned int j = 1; true; j <<= 1){
+   for(std::size_t i = 1; continue_loop; i <<= 1){
+      for(std::size_t j = 1; true; j <<= 1){
          void *ptr = a.allocate_aligned(i-1, j, std::nothrow);
          if(!ptr){
             if(j == 1)
                continue_loop = false;
             break;
          }
-         
+
          if(((std::size_t)ptr & (j - 1)) != 0)
             return false;
+         std::memset(ptr, 0xFF, i - 1);
          a.deallocate(ptr);
          if(!a.all_memory_deallocated() || !a.check_sanity()){
             return false;
@@ -428,7 +445,7 @@ bool test_continuous_aligned_allocation(Allocator &a)
    //and then deallocate it
    bool continue_loop = true;
    for(unsigned i = 1; continue_loop && i; i <<= 1){
-      for(unsigned int j = 1; j; j <<= 1){
+      for(std::size_t j = 1; j; j <<= 1){
          for(bool any_allocated = false; 1;){
             void *ptr = a.allocate_aligned(i-1, j, std::nothrow);
             buffers.push_back(ptr);
@@ -441,12 +458,12 @@ bool test_continuous_aligned_allocation(Allocator &a)
             else{
                any_allocated = true;
             }
-            
+
             if(((std::size_t)ptr & (j - 1)) != 0)
                return false;
          }
          //Deallocate all
-         for(unsigned int k = (int)buffers.size(); k--;){
+         for(std::size_t k = buffers.size(); k--;){
             a.deallocate(buffers[k]);
          }
          buffers.clear();
@@ -468,7 +485,7 @@ bool test_clear_free_memory(Allocator &a)
    std::vector<void*> buffers;
 
    //Allocate memory
-   for(int i = 0; true; ++i){
+   for(std::size_t i = 0; true; ++i){
       void *ptr = a.allocate(i, std::nothrow);
       if(!ptr)
          break;
@@ -478,12 +495,12 @@ bool test_clear_free_memory(Allocator &a)
    }
 
    //Mark it
-   for(int i = 0, max = buffers.size(); i < max; ++i){
+   for(std::size_t i = 0, max = buffers.size(); i < max; ++i){
       std::memset(buffers[i], 1, i);
    }
 
    //Deallocate all
-   for(int j = (int)buffers.size()
+   for(std::size_t j = buffers.size()
       ;j--
       ;){
       a.deallocate(buffers[j]);
@@ -502,7 +519,7 @@ bool test_clear_free_memory(Allocator &a)
    //Now test all allocated memory is zero
    //Allocate memory
    const char *first_addr = 0;
-   for(int i = 0; true; ++i){
+   for(std::size_t i = 0; true; ++i){
       void *ptr = a.allocate(i, std::nothrow);
       if(!ptr)
          break;
@@ -512,17 +529,19 @@ bool test_clear_free_memory(Allocator &a)
       std::size_t memsize = a.size(ptr);
       buffers.push_back(ptr);
 
-      for(int j = 0; j < (int)memsize; ++j){
+      for(std::size_t j = 0; j < memsize; ++j){
          if(static_cast<char*>((char*)ptr)[j]){
             std::cout << "Zero memory test failed. in buffer " << i
-                      << " byte " << j << " first address " << (void*) first_addr << " offset " << ((char*)ptr+j) - (char*)first_addr << " memsize: " << memsize << std::endl;
+                      << " byte " << j << " first address " << (const void*) first_addr
+                      << " offset " << ((const char*)ptr+j) - (const char*)first_addr
+                      << " memsize: " << memsize << std::endl;
             return false;
          }
       }
    }
 
    //Deallocate all
-   for(int j = (int)buffers.size()
+   for(std::size_t j = buffers.size()
       ;j--
       ;){
       a.deallocate(buffers[j]);
@@ -564,7 +583,7 @@ bool test_grow_shrink_to_fit(Allocator &a)
       return false;
 
    //Allocate memory
-   for(int i = 0; true; ++i){
+   for(std::size_t i = 0; true; ++i){
       void *ptr = a.allocate(i, std::nothrow);
       if(!ptr)
          break;
@@ -575,7 +594,7 @@ bool test_grow_shrink_to_fit(Allocator &a)
 
    //Now deallocate the half of the blocks
    //so expand maybe can merge new free blocks
-   for(int i = 0, max = (int)buffers.size()
+   for(std::size_t i = 0, max = buffers.size()
       ;i < max
       ;++i){
       if(i%2){
@@ -587,14 +606,14 @@ bool test_grow_shrink_to_fit(Allocator &a)
    //Deallocate the rest of the blocks
 
    //Deallocate it in non sequential order
-   for(int j = 0, max = (int)buffers.size()
+   for(std::size_t j = 0, max = buffers.size()
       ;j < max
       ;++j){
-      int pos = (j%5)*((int)buffers.size())/4;
-      if(pos == int(buffers.size()))
+      std::size_t pos = (j%5)*(buffers.size())/4;
+      if(pos == buffers.size())
          --pos;
       a.deallocate(buffers[pos]);
-      buffers.erase(buffers.begin()+pos);
+      buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
       typename Allocator::size_type old_free = a.get_free_memory();
       a.shrink_to_fit();
       if(!a.check_sanity())   return false;
@@ -645,11 +664,11 @@ bool test_many_equal_allocation(Allocator &a)
       std::vector<void*> buffers2;
 
       //Allocate buffers with extra memory
-      for(int i = 0; true; ++i){
+      for(std::size_t i = 0; true; ++i){
          void *ptr = a.allocate(i, std::nothrow);
          if(!ptr)
             break;
-		 std::size_t size = a.size(ptr);
+       std::size_t size = a.size(ptr);
          std::memset(ptr, 0, size);
          if(!a.check_sanity())
             return false;
@@ -658,7 +677,7 @@ bool test_many_equal_allocation(Allocator &a)
 
       //Now deallocate the half of the blocks
       //so expand maybe can merge new free blocks
-      for(int i = 0, max = (int)buffers2.size()
+      for(std::size_t i = 0, max = buffers2.size()
          ;i < max
          ;++i){
          if(i%2){
@@ -672,15 +691,15 @@ bool test_many_equal_allocation(Allocator &a)
 
       typedef typename Allocator::multiallocation_chain multiallocation_chain;
       std::vector<void*> buffers;
-      for(int i = 0; true; ++i){
-         multiallocation_chain chain(a.allocate_many(i+1, (i+1)*2, std::nothrow));
+      for(std::size_t i = 0; true; ++i){
+         multiallocation_chain chain;
+         a.allocate_many(std::nothrow, i+1, (i+1)*2, chain);
          if(chain.empty())
             break;
 
          typename multiallocation_chain::size_type n = chain.size();
          while(!chain.empty()){
-            buffers.push_back(ipcdetail::to_raw_pointer(chain.front()));
-            chain.pop_front();
+            buffers.push_back(ipcdetail::to_raw_pointer(chain.pop_front()));
          }
          if(n != std::size_t((i+1)*2))
             return false;
@@ -692,7 +711,7 @@ bool test_many_equal_allocation(Allocator &a)
       switch(t){
          case DirectDeallocation:
          {
-            for(int j = 0, max = (int)buffers.size()
+            for(std::size_t j = 0, max = buffers.size()
                ;j < max
                ;++j){
                a.deallocate(buffers[j]);
@@ -701,7 +720,7 @@ bool test_many_equal_allocation(Allocator &a)
          break;
          case InverseDeallocation:
          {
-            for(int j = (int)buffers.size()
+            for(std::size_t j = buffers.size()
                ;j--
                ;){
                a.deallocate(buffers[j]);
@@ -710,12 +729,12 @@ bool test_many_equal_allocation(Allocator &a)
          break;
          case MixedDeallocation:
          {
-            for(int j = 0, max = (int)buffers.size()
+            for(std::size_t j = 0, max = buffers.size()
                ;j < max
                ;++j){
-               int pos = (j%4)*((int)buffers.size())/4;
+               std::size_t pos = (j%4)*(buffers.size())/4;
                a.deallocate(buffers[pos]);
-               buffers.erase(buffers.begin()+pos);
+               buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
             }
          }
          break;
@@ -726,15 +745,15 @@ bool test_many_equal_allocation(Allocator &a)
       //Deallocate the rest of the blocks
 
       //Deallocate it in non sequential order
-      for(int j = 0, max = (int)buffers2.size()
+      for(std::size_t j = 0, max = buffers2.size()
          ;j < max
          ;++j){
-         int pos = (j%4)*((int)buffers2.size())/4;
+         std::size_t pos = (j%4)*(buffers2.size())/4;
          a.deallocate(buffers2[pos]);
-         buffers2.erase(buffers2.begin()+pos);
+         buffers2.erase(buffers2.begin()+std::ptrdiff_t(pos));
       }
 
-      bool ok = free_memory == a.get_free_memory() && 
+      bool ok = free_memory == a.get_free_memory() &&
                a.all_memory_deallocated() && a.check_sanity();
       if(!ok)  return ok;
    }
@@ -761,18 +780,18 @@ bool test_many_different_allocation(Allocator &a)
       std::vector<void*> buffers2;
 
       //Allocate buffers with extra memory
-      for(int i = 0; true; ++i){
+      for(std::size_t i = 0; true; ++i){
          void *ptr = a.allocate(i, std::nothrow);
          if(!ptr)
             break;
-		 std::size_t size = a.size(ptr);
+       std::size_t size = a.size(ptr);
          std::memset(ptr, 0, size);
          buffers2.push_back(ptr);
       }
 
       //Now deallocate the half of the blocks
       //so expand maybe can merge new free blocks
-      for(int i = 0, max = (int)buffers2.size()
+      for(std::size_t i = 0, max = buffers2.size()
          ;i < max
          ;++i){
          if(i%2){
@@ -782,14 +801,14 @@ bool test_many_different_allocation(Allocator &a)
       }
 
       std::vector<void*> buffers;
-      for(int i = 0; true; ++i){
-         multiallocation_chain chain(a.allocate_many(requested_sizes, ArraySize, 1, std::nothrow));
+      while(true){
+         multiallocation_chain chain;
+         a.allocate_many(std::nothrow, requested_sizes, ArraySize, 1, chain);
          if(chain.empty())
             break;
          typename multiallocation_chain::size_type n = chain.size();
          while(!chain.empty()){
-            buffers.push_back(ipcdetail::to_raw_pointer(chain.front()));
-            chain.pop_front();
+            buffers.push_back(ipcdetail::to_raw_pointer(chain.pop_front()));
          }
          if(n != ArraySize)
             return false;
@@ -798,7 +817,7 @@ bool test_many_different_allocation(Allocator &a)
       switch(t){
          case DirectDeallocation:
          {
-            for(int j = 0, max = (int)buffers.size()
+            for(std::size_t j = 0, max = buffers.size()
                ;j < max
                ;++j){
                a.deallocate(buffers[j]);
@@ -807,7 +826,7 @@ bool test_many_different_allocation(Allocator &a)
          break;
          case InverseDeallocation:
          {
-            for(int j = (int)buffers.size()
+            for(std::size_t j = buffers.size()
                ;j--
                ;){
                a.deallocate(buffers[j]);
@@ -816,12 +835,12 @@ bool test_many_different_allocation(Allocator &a)
          break;
          case MixedDeallocation:
          {
-            for(int j = 0, max = (int)buffers.size()
+            for(std::size_t j = 0, max = buffers.size()
                ;j < max
                ;++j){
-               int pos = (j%4)*((int)buffers.size())/4;
+               std::size_t pos = (j%4)*(buffers.size())/4;
                a.deallocate(buffers[pos]);
-               buffers.erase(buffers.begin()+pos);
+               buffers.erase(buffers.begin()+std::ptrdiff_t(pos));
             }
          }
          break;
@@ -832,15 +851,15 @@ bool test_many_different_allocation(Allocator &a)
       //Deallocate the rest of the blocks
 
       //Deallocate it in non sequential order
-      for(int j = 0, max = (int)buffers2.size()
+      for(std::size_t j = 0, max = buffers2.size()
          ;j < max
          ;++j){
-         int pos = (j%4)*((int)buffers2.size())/4;
+         std::size_t pos = (j%4)*(buffers2.size())/4;
          a.deallocate(buffers2[pos]);
-         buffers2.erase(buffers2.begin()+pos);
+         buffers2.erase(buffers2.begin()+std::ptrdiff_t(pos));
       }
 
-      bool ok = free_memory == a.get_free_memory() && 
+      bool ok = free_memory == a.get_free_memory() &&
                a.all_memory_deallocated() && a.check_sanity();
       if(!ok)  return ok;
    }
@@ -852,9 +871,11 @@ bool test_many_different_allocation(Allocator &a)
 template<class Allocator>
 bool test_many_deallocation(Allocator &a)
 {
+   typedef typename Allocator::multiallocation_chain multiallocation_chain;
+
    typedef typename Allocator::multiallocation_chain multiallocation_chain;
    const std::size_t ArraySize = 11;
-   vector<multiallocation_chain> buffers;
+   boost::container::vector<multiallocation_chain> buffers;
    typename Allocator::size_type requested_sizes[ArraySize];
    for(std::size_t i = 0; i < ArraySize; ++i){
       requested_sizes[i] = 4*i;
@@ -862,34 +883,36 @@ bool test_many_deallocation(Allocator &a)
    typename Allocator::size_type free_memory = a.get_free_memory();
 
    {
-      for(int i = 0; true; ++i){
-         multiallocation_chain chain = a.allocate_many(requested_sizes, ArraySize, 1, std::nothrow);
+      while(true){
+         multiallocation_chain chain;
+         a.allocate_many(std::nothrow, requested_sizes, ArraySize, 1, chain);
          if(chain.empty())
             break;
          buffers.push_back(boost::move(chain));
       }
-      for(int i = 0, max = (int)buffers.size(); i != max; ++i){
-         a.deallocate_many(boost::move(buffers[i]));
+      for(std::size_t i = 0, max = buffers.size(); i != max; ++i){
+         a.deallocate_many(buffers[i]);
       }
       buffers.clear();
-      bool ok = free_memory == a.get_free_memory() && 
+      bool ok = free_memory == a.get_free_memory() &&
                a.all_memory_deallocated() && a.check_sanity();
       if(!ok)  return ok;
    }
 
    {
-      for(int i = 0; true; ++i){
-         multiallocation_chain chain(a.allocate_many(i*4, ArraySize, std::nothrow));
+      for(std::size_t i = 0; true; ++i){
+         multiallocation_chain chain;
+         a.allocate_many(std::nothrow, i*4, ArraySize, chain);
          if(chain.empty())
             break;
          buffers.push_back(boost::move(chain));
       }
-      for(int i = 0, max = (int)buffers.size(); i != max; ++i){
-         a.deallocate_many(boost::move(buffers[i]));
+      for(std::size_t i = 0, max = buffers.size(); i != max; ++i){
+         a.deallocate_many(buffers[i]);
       }
       buffers.clear();
 
-      bool ok = free_memory == a.get_free_memory() && 
+      bool ok = free_memory == a.get_free_memory() &&
                a.all_memory_deallocated() && a.check_sanity();
       if(!ok)  return ok;
    }
