@@ -1,11 +1,17 @@
 #ifndef BOOST_PP_IS_ITERATING
+    ///////////////////////////////////////////////////////////////////////////////
+    //  Copyright 2008 Eric Niebler. Distributed under the Boost
+    //  Software License, Version 1.0. (See accompanying file
+    //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+    //
+    // This example contains a full-featured reimplementation of the old,
+    // now-deprecated Boost Lambda Library (BLL) on top of Boost.Proto. It
+    // is necessarily complex to accomodate all the quirks and inconsistencies
+    // of that old library, but it is a good example of how to build a
+    // complete and full-featured EDLS using Proto.
     #ifndef BOOST_LAMBDA_HPP_EAN_04_19_2008
     #define BOOST_LAMBDA_HPP_EAN_04_19_2008
 
-    #define BOOST_MPL_LIMIT_METAFUNCTION_ARITY 10
-    #define BOOST_PROTO_MAX_ARITY 10
-    #define BOOST_PROTO_MAX_FUNCTION_CALL_ARITY 5
-
     #include <iosfwd>
     #include <typeinfo>
     #include <algorithm>
@@ -18,7 +24,6 @@
     #include <boost/mpl/next_prior.hpp>
     #include <boost/mpl/min_max.hpp>
     #include <boost/mpl/assert.hpp>
-    #include <boost/mpl/apply_wrap.hpp>
     #include <boost/preprocessor.hpp>
     #include <boost/utility/enable_if.hpp>
     #include <boost/utility/result_of.hpp>
@@ -27,9 +32,6 @@
     #include <boost/type_traits/remove_reference.hpp>
     #include <boost/type_traits/remove_const.hpp>
     #include <boost/type_traits/is_same.hpp>
-    #include <boost/type_traits/is_abstract.hpp>
-    #include <boost/type_traits/is_array.hpp>
-    #include <boost/type_traits/is_function.hpp>
     #include <boost/proto/proto.hpp>
 
     #ifndef BOOST_LAMBDA_MAX_ARITY
@@ -57,7 +59,7 @@
             struct rethrow_ {};
             struct switch_ {};
             struct default_ {};
-            template<int I> struct case_ { BOOST_STATIC_CONSTANT(int, value = I); };
+            template<int I> struct case_ { static const int value = I; };
             template<typename E> struct catch_ { typedef E exception_type; };
             struct catch_all_ { typedef catch_all_ exception_type; };
         };
@@ -70,7 +72,7 @@
             typedef placeholder<Int> type;
             typedef placeholder<typename Int::next> next;
             typedef placeholder<typename Int::prior> prior;
-            BOOST_STATIC_CONSTANT(value_type, value = Int::value);
+            static const value_type value = Int::value;
 
             friend std::ostream &operator<<(std::ostream &sout, placeholder)
             {
@@ -103,30 +105,13 @@
               , proto::otherwise<proto::fold<proto::_, mpl::true_(), mpl::and_<proto::_state, IsNullary>()> >
             >
         {};
-        
-        struct at : proto::callable
-        {
-            template<class Sig>
-            struct result;
-
-            template<class This, class Cont, class Int>
-            struct result<This(Cont, Int)>
-              : fusion::result_of::at<
-                    typename remove_reference<Cont>::type
-                  , typename remove_reference<Int>::type
-                >
-            {};
-
-            template<typename Cont, typename Int>
-            typename fusion::result_of::at<Cont, Int>::type
-            operator ()(Cont &cont, Int const &) const
-            {
-                return fusion::at<Int>(cont);
-            }
-        };
 
         struct Eval;
 
+        template<typename Expr, typename State, typename Data>
+        typename boost::result_of<Eval(Expr&, State&, Data&)>::type
+        eval_lambda(Expr& e, State& s, Data& d);
+
         struct EvalWhile : proto::transform<EvalWhile>
         {
             template<typename Expr, typename State, typename Data>
@@ -140,9 +125,9 @@
                   , typename impl::data_param data
                 ) const
                 {
-                    while(Eval()(proto::left(expr), state, data))
+                    while(eval_lambda(proto::left(expr), state, data))
                     {
-                        Eval()(proto::right(expr), state, data);
+                        eval_lambda(proto::right(expr), state, data);
                     }
                     return result_type();
                 }
@@ -164,9 +149,9 @@
                 {
                     do
                     {
-                        Eval()(proto::child_c<0>(expr), state, data);
+                        eval_lambda(proto::child_c<0>(expr), state, data);
                     }
-                    while(Eval()(proto::child_c<1>(expr), state, data));
+                    while(eval_lambda(proto::child_c<1>(expr), state, data));
 
                     return result_type();
                 }
@@ -186,11 +171,11 @@
                   , typename impl::data_param data
                 ) const
                 {
-                    for(Eval()(proto::child_c<0>(expr), state, data)
-                      ; Eval()(proto::child_c<1>(expr), state, data)
-                      ; Eval()(proto::child_c<2>(expr), state, data))
+                    for(eval_lambda(proto::child_c<0>(expr), state, data)
+                      ; eval_lambda(proto::child_c<1>(expr), state, data)
+                      ; eval_lambda(proto::child_c<2>(expr), state, data))
                     {
-                        Eval()(proto::child_c<3>(expr), state, data);
+                        eval_lambda(proto::child_c<3>(expr), state, data);
                     }
                     return result_type();
                 }
@@ -210,9 +195,9 @@
                   , typename impl::data_param data
                 ) const
                 {
-                    if(Eval()(proto::left(expr), state, data))
+                    if(eval_lambda(proto::left(expr), state, data))
                     {
-                        Eval()(proto::right(expr), state, data);
+                        eval_lambda(proto::right(expr), state, data);
                     }
                     return result_type();
                 }
@@ -232,13 +217,13 @@
                   , typename impl::data_param data
                 ) const
                 {
-                    if(Eval()(proto::child_c<0>(expr), state, data))
+                    if(eval_lambda(proto::child_c<0>(expr), state, data))
                     {
-                        Eval()(proto::child_c<1>(expr), state, data);
+                        eval_lambda(proto::child_c<1>(expr), state, data);
                     }
                     else
                     {
-                        Eval()(proto::child_c<2>(expr), state, data);
+                        eval_lambda(proto::child_c<2>(expr), state, data);
                     }
                     return result_type();
                 }
@@ -269,10 +254,10 @@
         {
             template<typename Expr, typename State, typename Data, long Arity, typename BackTag>
             struct impl2;
-            
+
             #define M0(Z, N, DATA)                                                                  \
                 case proto::tag_of<typename proto::result_of::child_c<Expr, N>::type>::type::value: \
-                    Eval()(proto::child_c<N>(expr), state, data);                                   \
+                    eval_lambda(proto::child_c<N>(expr), state, data);                              \
                     break;                                                                          \
                     /**/
 
@@ -289,7 +274,7 @@
                   , typename impl2::data_param data                                                 \
                 ) const                                                                             \
                 {                                                                                   \
-                    switch(Eval()(proto::child_c<0>(expr), state, data))                            \
+                    switch(eval_lambda(proto::child_c<0>(expr), state, data))                       \
                     {                                                                               \
                         BOOST_PP_REPEAT_FROM_TO_ ## Z(1, N, M0, ~)                                  \
                     default:                                                                        \
@@ -310,11 +295,11 @@
                   , typename impl2::data_param data                                                 \
                 ) const                                                                             \
                 {                                                                                   \
-                    switch(Eval()(proto::child_c<0>(expr), state, data))                            \
+                    switch(eval_lambda(proto::child_c<0>(expr), state, data))                       \
                     {                                                                               \
                         BOOST_PP_REPEAT_FROM_TO_ ## Z(1, BOOST_PP_DEC(N), M0, ~)                    \
                     default:;                                                                       \
-                        Eval()(proto::child_c<BOOST_PP_DEC(N)>(expr), state, data);                 \
+                        eval_lambda(proto::child_c<BOOST_PP_DEC(N)>(expr), state, data);            \
                         break;                                                                      \
                     }                                                                               \
                 }                                                                                   \
@@ -351,23 +336,23 @@
                 throw e;
             }
         };
-        
+
         struct unwrap_ref : proto::callable
         {
             template<typename Sig>
             struct result;
-            
+
             template<typename This, typename T>
             struct result<This(reference_wrapper<T>)>
             {
-                typedef T &type;                
+                typedef T &type;
             };
-            
+
             template<typename This, typename T>
             struct result<This(T &)>
               : result<This(T)>
             {};
-            
+
             template<typename T>
             T &operator()(reference_wrapper<T> const &ref) const
             {
@@ -430,7 +415,7 @@
           : proto::or_<
                 proto::when<
                     proto::terminal<placeholder<proto::_> >
-                  , at(proto::_data, proto::_value)
+                  , proto::functional::at(proto::_data, proto::_value)
                 >
               , proto::when<
                     proto::terminal<exception_placeholder>
@@ -459,6 +444,13 @@
           : proto::switch_<Cases>
         {};
 
+        template<typename Expr, typename State, typename Data>
+        typename boost::result_of<Eval(Expr&, State&, Data&)>::type
+        eval_lambda(Expr& e, State& s, Data& d)
+        {
+            return Eval()(e, s, d);
+        }
+
         // Use a grammar to disable Proto's assignment operator overloads.
         // We'll define our own because we want (x+=_1) to store x by
         // reference. (In all other cases, variables are stored by value
@@ -483,66 +475,32 @@
         template<> struct AssignOps::case_<proto::tag::bitwise_or_assign>   : proto::not_<proto::_> {};
         template<> struct AssignOps::case_<proto::tag::bitwise_xor_assign>  : proto::not_<proto::_> {};
 
-        namespace exprns_
-        {
-            template<typename Expr>
-            struct llexpr;
-        }
+        template<typename Expr>
+        struct llexpr;
 
-        using exprns_::llexpr;
-
-        template<typename T>
-        struct is_stream
-        {
-        private:
-            static T &t;
-            typedef char yes_type;
-            typedef char (&no_type)[2];
-            static no_type test_is_stream(...);
-            template<typename Char, typename Traits>
-            static yes_type test_is_stream(std::basic_istream<Char, Traits> &);
-            template<typename Char, typename Traits>
-            static yes_type test_is_stream(std::basic_ostream<Char, Traits> &);
-        public:
-            typedef bool value_type;
-            BOOST_STATIC_CONSTANT(bool, value = sizeof(yes_type) == sizeof(test_is_stream(t)));
-            typedef mpl::bool_<value> type;
-        };
-
-        // These terminal types are always stored by reference
-        template<typename Value>
-        struct store_by_ref
-          : mpl::or_<
-                is_abstract<Value>
-              , is_array<Value>
-              , is_function<Value>
-              , is_stream<Value>
-            >
-        {};
-
-        // Wrap expressions in lambda::llexpr<>, and hold children nodes
-        // and some terminal types by value.
+        // Wrap expressions in lambda::llexpr<>.
         struct Generator
-          : proto::or_<
-                proto::when<
-                    proto::and_<
-                        proto::terminal<proto::_>
-                      , proto::if_<store_by_ref<proto::_value>()>
-                    >
-                  , proto::pod_generator<llexpr>(proto::_)
-                >
-              , proto::otherwise<
-                    proto::compose_generators<
-                        proto::by_value_generator
-                      , proto::pod_generator<llexpr>
-                    >(proto::_)
-                >
-            >
+          : proto::pod_generator<llexpr>
         {};
 
+        // The domain for the lambda library.
         struct lldomain
-          : proto::domain<Generator, Grammar>
-        {};
+          : proto::domain<Generator, Grammar, proto::default_domain>
+        {
+            // Make all terminals and children held by value instead of by reference.
+            // Proto::domain<>::as_expr<> holds everything it can by value; the only
+            // exceptions are function types, abstract types, and iostreams.
+            template<typename T>
+            struct as_child
+              : proto_base_domain::as_expr<T>
+            {};
+
+            // The exception is arrays, which should still be held by reference
+            template<typename T, std::size_t N>
+            struct as_child<T[N]>
+              : proto_base_domain::as_child<T[N]>
+            {};
+        };
 
         template<typename Sig>
         struct llresult;
@@ -551,7 +509,7 @@
         struct llresult<This()>
           : mpl::if_c<
                 result_of<IsNullary(This &)>::type::value
-              , result_of<Eval(This &, no_exception_type const &, fusion::vector0 &)>
+              , result_of<Eval(This &, no_exception_type const &, fusion::vector0<> &)>
               , mpl::identity<void>
             >::type
         {};
@@ -586,8 +544,8 @@
             typename result<llexpr const()>::type
             operator()() const
             {
-                fusion::vector0 args;
-                return Eval()(*this, no_exception, args);
+                fusion::vector0<> args;
+                return eval_lambda(*this, no_exception, args);
             }
 
             #define M1(Z, N, _) ((0)(1))
@@ -602,7 +560,7 @@
                     BOOST_MPL_ASSERT_RELATION(result_of<Arity(Expr const &)>::type::value, <=, SIZE);       \
                     BOOST_PP_CAT(fusion::vector, SIZE)<BOOST_PP_SEQ_FOR_EACH_I_R(R, M5, ~, PRODUCT)> args   \
                         (BOOST_PP_SEQ_FOR_EACH_I_R(R, M6, ~, PRODUCT));                                     \
-                    return Eval()(*this, no_exception, args);                                               \
+                    return eval_lambda(*this, no_exception, args);                                          \
                 }                                                                                           \
                 /**/
 
@@ -622,7 +580,7 @@
 
             #define C1 const
 
-            #define BOOST_PP_ITERATION_PARAMS_1 (3, (1, BOOST_LAMBDA_MAX_ARITY, "./lambda.hpp"))
+            #define BOOST_PP_ITERATION_PARAMS_1 (3, (1, BOOST_LAMBDA_MAX_ARITY, "lambda.hpp"))
             #include BOOST_PP_ITERATE()
 
             #undef C0
@@ -654,7 +612,7 @@
         {
             template<typename Sig>
             struct result;
-            
+
             template<typename This, typename T>
             struct result<This(T &)>
             {
@@ -746,7 +704,7 @@
         {
             typedef llexpr<typename proto::terminal<T &>::type> type;
         };
-        
+
         template<typename T>
         llexpr<typename proto::terminal<T &>::type> const
         var(T &t)
@@ -777,7 +735,7 @@
         {
             typedef llexpr<typename proto::terminal<T const &>::type> type;
         };
-        
+
         template<typename T>
         llexpr<typename proto::terminal<T const &>::type> const
         constant_ref(T const &t)
@@ -1143,7 +1101,7 @@
         {
             return t;
         }
-        
+
         #define M1(N, typename_A, A_const_ref, A_const_ref_a, ref_a)                                \
         template<typename_A(N)>                                                                     \
         typename proto::result_of::make_expr<                                                       \
@@ -1508,7 +1466,7 @@
                 }
                 catch(Exception const &e)
                 {
-                    return static_cast<result_type>(Eval()(this->head, e, data));
+                    return static_cast<result_type>(eval_lambda(this->head, e, data));
                 }
             }
 
@@ -1524,7 +1482,7 @@
                 }
                 catch(...)
                 {
-                    return static_cast<result_type>(Eval()(this->head, tag::catch_all_(), data));
+                    return static_cast<result_type>(eval_lambda(this->head, tag::catch_all_(), data));
                 }
             }
 
@@ -1550,7 +1508,7 @@
             typename result_of<Eval(Head const &, State, Data)>::type
             operator()(State const &state, Data &data) const
             {
-                return Eval()(this->head, state, data);
+                return eval_lambda(this->head, state, data);
             }
 
         private:
@@ -1742,7 +1700,7 @@
         }
 
     }}
-    
+
     namespace boost
     {
         template<typename Expr>
