@@ -1,6 +1,7 @@
+#!/usr/bin/env python3
 # Copyright Stefan Seefeld 2006. Distributed under the Boost
 # Software License, Version 1.0. (See accompanying
 # file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-print 'Hello World !'
+print('Hello World !')
 number = 42
