@@ -1,10 +1,9 @@
-//  Copyright (c) 2001-2010 Hartmut Kaiser
+//  Copyright (c) 2001-2011 Hartmut Kaiser
 // 
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying 
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#include <boost/config/warning_disable.hpp>
-#include <boost/detail/lightweight_test.hpp>
+#include <boost/spirit/include/karma_buffer.hpp>
 
 #include <boost/spirit/include/karma_numeric.hpp>
 #include <boost/spirit/include/karma_generate.hpp>
@@ -31,6 +30,7 @@ main()
         v.push_back(1.0);
         v.push_back(2.0);
         BOOST_TEST(test("[1.02.0]", buffer['[' << +double_ << ']'], v));
+        BOOST_TEST(test("[1.02.0]", buffer[buffer['[' << +double_ << ']']], v));
     }
 
     {
